@@ -1,19 +1,20 @@
 from biblemate.core.systems import *
+from biblemate.uba.dialogs import *
 from biblemate.ui.text_area import getTextArea
 from biblemate.ui.info import get_banner
-from biblemate.ui.selection_dialog import TerminalModeDialogs
 from biblemate import config, BIBLEMATE_VERSION, AGENTMAKE_CONFIG, BIBLEMATEDATA, fix_string, write_user_config
 from biblemate.uba.api import run_uba_api, DEFAULT_MODULES
 from pathlib import Path
 import urllib.parse
-import asyncio, re, os, subprocess, click, gdown, pprint, argparse, json, zipfile, warnings
+import asyncio, re, os, subprocess, click, gdown, pprint, argparse, json, zipfile, warnings, sys
 from copy import deepcopy
 from alive_progress import alive_bar
 from fastmcp import Client
 from fastmcp.client.transports import StreamableHttpTransport
 from agentmake.plugins.uba.lib.BibleBooks import BibleBooks
-from agentmake import agentmake, getOpenCommand, getDictionaryOutput, edit_file, edit_configurations, readTextFile, writeTextFile, getCurrentDateTime, AGENTMAKE_USER_DIR, USER_OS, DEVELOPER_MODE, DEFAULT_AI_BACKEND
+from agentmake import agentmake, getOpenCommand, getDictionaryOutput, edit_file, edit_configurations, readTextFile, writeTextFile, getCurrentDateTime, AGENTMAKE_USER_DIR, USER_OS, DEVELOPER_MODE, DEFAULT_AI_BACKEND, DEFAULT_TEXT_EDITOR
 #from agentmake.utils.handle_text import set_log_file_max_lines
+from agentmake.etextedit import launch_async
 from agentmake.utils.manage_package import getPackageLatestVersion
 from rich.console import Console
 from rich.panel import Panel
@@ -23,6 +24,8 @@ from rich.markdown import Markdown
 from rich.progress import Progress, SpinnerColumn, TextColumn
 from rich.terminal_theme import MONOKAI
 from prompt_toolkit.shortcuts import set_title, clear_title
+from prompt_toolkit.completion import PathCompleter
+from packaging import version
 if not USER_OS == "Windows":
     import readline  # for better input experience
 
@@ -34,7 +37,7 @@ log_file = os.path.join(log_path, "requests")
 set_log_file_max_lines(log_file, config.max_log_lines)"""
 
 # set window title
-set_title(f"BibleMate AI [{BIBLEMATE_VERSION}]")
+set_title(f"BibleMate AI")
 
 parser = argparse.ArgumentParser(description = f"""BibleMate AI {BIBLEMATE_VERSION} CLI options""")
 # global options
@@ -50,6 +53,14 @@ parser.add_argument("-t", "--token", action="store", dest="token", help="specify
 parser.add_argument("-mcp", "--mcp", action="store", dest="mcp", help=f"specify a custom MCP server to use, e.g. 'http://127.0.0.1:{config.mcp_port}/mcp/'; applicable to command `biblemate` only")
 parser.add_argument("-p", "--port", action="store", dest="port", help=f"specify a port for the MCP server to use, e.g. {config.mcp_port}; applicable to command `biblematemcp` only")
 args = parser.parse_args()
+
+if not sys.stdin.isatty():
+    stdin_text = sys.stdin.read()
+    if args.default:
+        args.default.append(stdin_text)
+    else:
+        args.default = [stdin_text]
+
 # write to the `config.py` file temporarily for the MCP server to pick it up
 if args.backend:
     with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), "config.py"), "a", encoding="utf-8") as fileObj:
@@ -169,41 +180,70 @@ def display_info(console, info):
         border_style="bright_blue",
         box=box.ROUNDED,
         style="on grey11" if isinstance(info, str) else "",
-        padding=(1, 1)
+        #padding=(1 if isinstance(info, str) else 0, 1) # (0, 1) by default
     )
     console.print(info_panel)
     console.print()
 
-def backup_conversation(messages, master_plan, console=None):
+def backup_conversation(messages, master_plan, console=None, storage_path=None):
     """Backs up the current conversation to the user's directory."""
     if len(messages) > len(DEFAULT_MESSAGES):
         # determine storage path
-        if console:
-            timestamp = getCurrentDateTime()
-            storagePath = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "chats", timestamp)
-        else:
-            storagePath = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp")
+        if not storage_path:
+            if console:
+                timestamp = getCurrentDateTime()
+                storage_path = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "chats", timestamp)
+            else:
+                storage_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp")
         # create directory if not exists
-        if not os.path.isdir(storagePath):
-            Path(storagePath).mkdir(parents=True, exist_ok=True)
+        if not os.path.isdir(storage_path):
+            Path(storage_path).mkdir(parents=True, exist_ok=True)
         # Save full conversation
-        conversation_file = os.path.join(storagePath, "conversation.py")
+        conversation_file = os.path.join(storage_path, "conversation.py")
         writeTextFile(conversation_file, pprint.pformat(messages))
         # Save master plan
-        writeTextFile(os.path.join(storagePath, "master_plan.md"), master_plan)
+        writeTextFile(os.path.join(storage_path, "master_plan.md"), master_plan)
         # Save markdown
-        markdown_file = os.path.join(storagePath, "conversation.md")
+        markdown_file = os.path.join(storage_path, "conversation.md")
         markdown_text = "\n\n".join(["```"+i["role"]+"\n"+i["content"]+"\n```" for i in messages if i.get("role", "") in ("user", "assistant")])
         writeTextFile(markdown_file, markdown_text)
         # Save html
         if console:
-            html_file = os.path.join(storagePath, "conversation.html")
+            html_file = os.path.join(storage_path, "conversation.html")
             console.save_html(html_file, inline_styles=True, theme=MONOKAI)
         # Inform users of the backup location
         if console:
-            info = f"Conversation saved to: {storagePath}\nReport saved to: {html_file}"
+            info = f"Conversation saved to: {storage_path}\nReport saved to: {html_file}"
             display_info(console, info)
 
+async def download_data(console, default=""):
+    file_ids ={
+        "bible.db": "1E6pDKfjUMhmMWjjazrg5ZcpH1RBD8qgW",
+        "collection.db": "1y4txzRzXTBty0aYfFgkWfz5qlHERrA17",
+        "dictionary.db": "1UxDKGEQa7UEIJ6Ggknx13Yt8XNvo3Ld3",
+        "encyclopedia.db": "1NLUBepvFd9UDxoGQyQ-IohmySjjeis2-",
+        "exlb.db": "1Hpo6iLSh5KzgR6IZ-c7KuML--A3nmP1-",
+    }
+    file_id = await DIALOGS.getValidOptions(
+        options=file_ids.keys(),
+        title="BibleMate Data Files",
+        text="Select a file:",
+        default=default,
+    )
+    if file_id:
+        output = os.path.join(BIBLEMATEDATA, file_id+".zip")
+        if os.path.isfile(output):
+            os.remove(output)
+        if os.path.isfile(output[:-4]):
+            os.remove(output[:-4])
+        gdown.download(id=file_ids[file_id], output=output)
+        with zipfile.ZipFile(output, 'r') as zip_ref:
+            zip_ref.extractall(BIBLEMATEDATA)
+        if os.path.isfile(output):
+            os.remove(output)
+        info = "Restart to make the changes effective!"
+        display_info(console, info)
+
 async def main_async():
 
     BIBLEMATE_STATIC_TOKEN = args.token if args.token else os.getenv("BIBLEMATE_STATIC_TOKEN")
@@ -230,12 +270,28 @@ async def main_async():
     console = Console(record=True)
     console.clear()
     console.print(get_banner(BIBLEMATE_VERSION))
-    dialogs = TerminalModeDialogs(None)
 
     async with client:
         tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema, resources, templates = await initialize_app(client)
-        resource_suggestions = json.loads(run_uba_api(".resources"))
-        resource_suggestions = [f"//bible/{i}/" for i in resource_suggestions["bibleListAbb"]]+[f"//commentary/{i}/" for i in resource_suggestions["commentaryListAbb"]]+[f"//encyclopedia/{i}/" for i in resource_suggestions["encyclopediaListAbb"]]+[f"//lexicon/{i}/" for i in resource_suggestions["lexiconList"]]
+        resource_suggestions_raw = json.loads(run_uba_api(".resources"))
+        # check if default modules are valid:
+        config_changed = False
+        if not config.default_bible in resource_suggestions_raw["bibleListAbb"]:
+            config.default_bible = "NET"
+            config_changed = True
+        if not config.default_commentary in resource_suggestions_raw["commentaryListAbb"]:
+            config.default_commentary = "CBSC"
+            config_changed = True
+        if not config.default_encyclopedia in resource_suggestions_raw["encyclopediaListAbb"]:
+            config.default_encyclopedia = "ISB"
+            config_changed = True
+        if not config.default_lexicon in resource_suggestions_raw["lexiconList"]:
+            config.default_lexicon = "Morphology"
+            config_changed = True
+        if config_changed:
+            write_user_config()
+        # format input suggestions
+        resource_suggestions = [f"//bible/{i}/" for i in resource_suggestions_raw["bibleListAbb"]]+[f"//commentary/{i}/" for i in resource_suggestions_raw["commentaryListAbb"]]+[f"//encyclopedia/{i}/" for i in resource_suggestions_raw["encyclopediaListAbb"]]+[f"//lexicon/{i}/" for i in resource_suggestions_raw["lexiconList"]]
         abbr = BibleBooks.abbrev["eng"]
         resource_suggestions += [abbr[str(book)][0] for book in range(1,67)]
 
@@ -292,6 +348,9 @@ async def main_async():
                 # Await the custom async progress bar that awaits the task.
                 await async_alive_bar(task)
 
+            if not APP_START and args.exit:
+                break
+
             if not len(messages) == len(DEFAULT_MESSAGES):
                 console.rule()
             elif APP_START:
@@ -299,7 +358,7 @@ async def main_async():
                 print()
                 # check for updates
                 latest_version = getPackageLatestVersion("biblemate")
-                if latest_version and str(latest_version).strip() != BIBLEMATE_VERSION:
+                if latest_version and latest_version > version.parse(BIBLEMATE_VERSION):
                     info = f"A new version of BibleMate AI is available: {latest_version} (you are using {BIBLEMATE_VERSION}).\nTo upgrade, close `BibleMate AI` first and run `pip install --upgrade biblemate`."
                     display_info(console, info)
                 # check connection
@@ -316,8 +375,42 @@ async def main_async():
             # note: `python3 -m rich.emoji` for checking emoji
             console.print("Enter your request :smiley: :" if len(messages) == len(DEFAULT_MESSAGES) else "Enter a follow-up request :flexed_biceps: :")
             action_list = {
-                ".new": "new conversation",
+                # general
+                ".ideas": "generate ideas for prompts to try",
                 ".exit": "exit current prompt",
+                # conversations
+                ".new": "new conversation",
+                ".trim": "trim conversation",
+                ".edit": "edit conversation",
+                ".import": "import conversation",
+                ".export": "export conversation",
+                ".backup": "backup conversation",
+                # UBA content
+                ".bible": "open bible verse",
+                ".chapter": "open bible chapter",
+                ".compare": "compare bible verse in different versions",
+                ".comparechapter": "compare bible chapter in different versions",
+                ".search": "search bible",
+                ".commentary": "open commentary",
+                ".dictionary": "search dictionary",
+                ".encyclopedia": "search encyclopedia",
+                ".lexicon": "search lexicon",
+                ".parallel": "search parallel passages",
+                ".promise": "search bible promises",
+                ".topic": "search bible topic",
+                ".name": "search bible name",
+                ".character": "search bible character",
+                ".location": "search bible location",
+                ".chronology": "open bible chronology",
+                ".defaultbible": "configure default bible",
+                ".defaultcommentary": "configure default commentary",
+                ".defaultencyclopedia": "configure default encyclopedia",
+                ".defaultlexicon": "configure default lexicon",
+                # resource information
+                ".tools": "list available tools",
+                ".plans": "list available plans",
+                ".resources": "list UniqueBible resources",
+                # configurations
                 ".backend": "configure backend",
                 ".steps": "configure the maximum number of steps allowed",
                 ".matches": "configure the maximum number of semantic matches",
@@ -325,26 +418,28 @@ async def main_async():
                 ".agent": "switch to agent mode",
                 ".partner": "switch to partner mode",
                 ".chat": "switch to chat mode",
-                ".tools": "list available tools",
-                ".plans": "list available plans",
-                ".resources": "list UniqueBible resources",
                 ".autosuggestions": "toggle auto input suggestions",
                 ".promptengineer": "toggle auto prompt engineering",
                 ".lite": "toggle lite context",
-                ".edit": "load the current conversation",
-                ".backup": "backup conversation",
-                ".load": "load a saved conversation",
-                ".open": "open a file or directory",
+                # file access
+                ".open": "open file or folder",
                 ".download": "download data files",
-                ".ideas": "generate ideas for prompts to try",
+                # help
                 ".help": "help page",
             }
             input_suggestions = list(action_list.keys())+["@ ", "@@ "]+[f"@{t} " for t in available_tools]+[f"{p} " for p in prompt_list]+[f"//{r}" for r in resources.keys()]+template_list+resource_suggestions
             if args.default:
-                user_request = " ".join(args.default)
+                user_request = " ".join(args.default).strip()
                 args.default = None # reset to avoid repeated use
+                display_info(console, user_request)
             else:
                 user_request = await getTextArea(input_suggestions=input_suggestions)
+            # luanch action menu
+            if user_request == ".":
+                select = await DIALOGS.getValidOptions(options=action_list.keys(), descriptions=[i.capitalize() for i in action_list.values()], title="Action Menu", text="Select an action:")
+                user_request = select if select else ""
+            if not user_request:
+                continue
             if user_request == ".ideas":
                 # Generate ideas for `prompts to try`
                 ideas = ""
@@ -371,11 +466,78 @@ async def main_async():
                     else:
                         display_content = resource_text
                     resource_description = resources.get(resource, "")
-                    info = Markdown(f"## Information about `{resource}`: `{resource.capitalize()}`\n\n{resource_description}\n\n{display_content}")
+                    info = Markdown(f"## Information - `{resource.capitalize()}`\n\n{resource_description}\n\n{display_content}")
                     display_info(console, info)
                 continue
 
             # run templates
+            if user_request == ".bible":
+                user_request = await uba_bible(options=resource_suggestions_raw["bibleListAbb"], descriptions=resource_suggestions_raw["bibleList"])
+            elif user_request == ".search":
+                user_request = await uba_search_bible(options=resource_suggestions_raw["bibleListAbb"], descriptions=resource_suggestions_raw["bibleList"])
+            elif user_request == ".chapter":
+                user_request = await uba_chapter(options=resource_suggestions_raw["bibleListAbb"], descriptions=resource_suggestions_raw["bibleList"])
+            elif user_request == ".compare":
+                user_request = await uba_compare(options=resource_suggestions_raw["bibleListAbb"], descriptions=resource_suggestions_raw["bibleList"])
+            elif user_request == ".comparechapter":
+                user_request = await uba_compare_chapter(options=resource_suggestions_raw["bibleListAbb"], descriptions=resource_suggestions_raw["bibleList"])
+            elif user_request == ".commentary":
+                user_request = await uba_commentary(options=resource_suggestions_raw["commentaryListAbb"], descriptions=resource_suggestions_raw["commentaryList"])
+            elif user_request == ".dictionary":
+                if not args.mcp and not "//dictionary/" in template_list:
+                    await download_data(console, default="dictionary.db")
+                    continue
+                else:
+                    user_request = await uba_dictionary()
+            elif user_request == ".parallel":
+                if not args.mcp and not "//parallel/" in template_list:
+                    await download_data(console, default="collection.db")
+                    continue
+                else:
+                    user_request = await uba_parallel()
+            elif user_request == ".promise":
+                if not args.mcp and not "//promise/" in template_list:
+                    await download_data(console, default="collection.db")
+                    continue
+                else:
+                    user_request = await uba_promise()
+            elif user_request == ".topic":
+                if not args.mcp and not "//topic/" in template_list:
+                    await download_data(console, default="exlb.db")
+                    continue
+                else:
+                    user_request = await uba_topic()
+            elif user_request == ".name":
+                if not args.mcp and not "//name/" in template_list:
+                    await download_data(console, default="exlb.db")
+                    continue
+                else:
+                    user_request = await uba_name()
+            elif user_request == ".character":
+                if not args.mcp and not "//character/" in template_list:
+                    await download_data(console, default="exlb.db")
+                    continue
+                else:
+                    user_request = await uba_character()
+            elif user_request == ".location":
+                if not args.mcp and not "//location/" in template_list:
+                    await download_data(console, default="exlb.db")
+                    continue
+                else:
+                    user_request = await uba_location()
+            elif user_request == ".encyclopedia":
+                if not args.mcp and not "//encyclopedia/" in template_list:
+                    await download_data(console, default="encyclopedia.db")
+                    continue
+                else:
+                    user_request = await uba_encyclopedia(options=resource_suggestions_raw["encyclopediaListAbb"], descriptions=resource_suggestions_raw["encyclopediaList"])
+            elif user_request == ".lexicon":
+                user_request = await uba_lexicon(options=resource_suggestions_raw["lexiconList"])
+            elif user_request == ".chronology":
+                user_request = "//uba/DATA:::Bible Chronology"
+            if not user_request:
+                continue
+
             if re.search(template_pattern, user_request):
                 user_request = urllib.parse.quote(user_request)
                 if user_request[2:].count("/") == 1:
@@ -393,7 +555,7 @@ async def main_async():
                     resource_content = resource_content[0].text
                     while resource_content.startswith("[") and resource_content.endswith("]"):
                         options = json.loads(resource_content)
-                        select = await dialogs.getValidOptions(
+                        select = await DIALOGS.getValidOptions(
                             options=options,
                             title="Multiple Matches",
                             text="Select one of them to continue:"
@@ -411,37 +573,53 @@ async def main_async():
                             {"role": "user", "content": f"Retrieve content from:\n\n{uri}"},
                             {"role": "assistant", "content": resource_content},
                         ]
-                        console.rule()
                         if resource_content == "Cancelled by user.":
                             info = resource_content
                             display_info(console, info)
                         else:
-                            console.print(Markdown(resource_content))
+                            info = Markdown(resource_content.strip())
+                            display_info(console, info)
                     continue
                 except Exception as e: # invalid uri
                     print(f"Error: {e}\n")
                     continue
-            
+
             # system command
+            if user_request.startswith(".open") or user_request.startswith(".import"):
+                cwd = os.getcwd()
             if user_request == ".open":
-                user_request = f".open {os.getcwd()}"
+                os.chdir(os.path.dirname(BIBLEMATEDATA))
+                open_item = await DIALOGS.getInputDialog(title="Open", text="Enter a file or folder path:", suggestions=PathCompleter())
+                if not open_item:
+                    open_item = os.getcwd()
+                user_request = f".open {open_item}"
+            elif user_request == ".import":
+                chats_path = os.path.join(os.path.dirname(BIBLEMATEDATA), "chats")
+                os.chdir(chats_path)
+                import_item = await DIALOGS.getInputDialog(title="Import", text="Enter a conversation file or folder path:", suggestions=PathCompleter())
+                if import_item:
+                    user_request = f".import {import_item}"
+                else:
+                    user_request = f".open {chats_path}"
             if user_request.startswith(".open ") and os.path.exists(os.path.expanduser(re.sub('''^['" ]*?([^'" ].+?)['" ]*?$''', r"\1", user_request[6:]))):
                 file_path = os.path.expanduser(re.sub('''^['" ]*?([^'" ].+?)['" ]*?$''', r"\1", user_request[6:]))
                 cmd = f'''{getOpenCommand()} "{file_path}"'''
                 with warnings.catch_warnings():
                     warnings.filterwarnings("ignore", category=ResourceWarning)
                     subprocess.Popen(cmd, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+                os.chdir(cwd)
                 continue
-            elif user_request.startswith(".load") and os.path.exists(os.path.expanduser(re.sub('''^['" ]*?([^'" ].+?)['" ]*?$''', r"\1", user_request[6:]))):
-                load_path = os.path.expanduser(re.sub('''^['" ]*?([^'" ].+?)['" ]*?$''', r"\1", user_request[6:]))
+            elif user_request.startswith(".import ") and os.path.exists(os.path.expanduser(re.sub('''^['" ]*?([^'" ].+?)['" ]*?$''', r"\1", user_request[8:]))):
+                load_path = os.path.expanduser(re.sub('''^['" ]*?([^'" ].+?)['" ]*?$''', r"\1", user_request[8:]))
                 try:
-                    # load conversation
+                    # import conversation
                     if os.path.isfile(load_path):
                         file_path = load_path
                     elif os.path.isdir(load_path) and os.path.isfile(os.path.join(load_path, "conversation.py")) and os.path.isfile(os.path.join(load_path, "master_plan.md")):
                         file_path = os.path.join(load_path, "conversation.py")
                     else:
                         print("Expected a file or a directory containing `conversation.py` and `master_plan.md`.")
+                        os.chdir(cwd)
                         continue
                     backup_conversation(messages, master_plan, console)
                     messages = [{"role": i["role"], "content": i["content"]} for i in eval(readTextFile(file_path)) if i.get("role", "") in ("user", "assistant")]
@@ -449,7 +627,7 @@ async def main_async():
                         messages.insert(0, {"role": "system", "content": DEFAULT_SYSTEM})
                     if messages[-1].get("role", "") == "user":
                         messages = messages[:-1]
-                    # load master plan
+                    # import master plan
                     if os.path.isdir(load_path):
                         master_plan = readTextFile(os.path.join(load_path, "master_plan.md"))
                         user_request = "[CONTINUE]"
@@ -465,10 +643,14 @@ async def main_async():
                                 console.print(Markdown(f"# {i['role']}\n\n{i['content']}"))
                     if os.path.isfile(load_path) or config.agent_mode is None:
                         # next user request
+                        os.chdir(cwd)
                         continue
                 except Exception as e:
                     print(f"Error: {e}\n")
+                    os.chdir(cwd)
                     continue
+            if user_request.startswith(".open") or user_request.startswith(".import"):
+                os.chdir(cwd)
 
             # predefined operations with `.` commands
             if user_request in action_list:
@@ -510,7 +692,7 @@ Viist https://github.com/eliranwong/biblemate
 - `Esc`: close input suggestion menu"""
                     display_info(console, Markdown(help_info))
                 elif user_request == ".tools":
-                    enabled_tools = await dialogs.getMultipleSelection(
+                    enabled_tools = await DIALOGS.getMultipleSelection(
                         default_values=available_tools,
                         options=master_available_tools,
                         title="Tool Options",
@@ -533,21 +715,66 @@ Viist https://github.com/eliranwong/biblemate
                     prompts_descriptions = [f"- `/{name}`: {description}" for name, description in prompts.items()]
                     info = Markdown("## Available Plans\n\n"+"\n".join(prompts_descriptions))
                     display_info(console, info)
-                elif user_request == ".edit":
+                elif user_request == ".export":
+                    cwd = os.getcwd()
+                    chats_path = os.path.join(os.path.dirname(BIBLEMATEDATA), "chats")
+                    if not os.path.isdir(chats_path):
+                        Path(chats_path).mkdir(parents=True, exist_ok=True)
+                    os.chdir(chats_path)
+                    export_item = await DIALOGS.getInputDialog(title="Export", text="Enter a name or path:", default=chats_path, suggestions=PathCompleter())
+                    if export_item:
+                        export_item_parent = os.path.dirname(export_item)
+                        if not export_item_parent:
+                            storage_path = os.path.join(chats_path, export_item)
+                        elif os.path.isdir(export_item_parent):
+                            storage_path = export_item
+                        else:
+                            storage_path = os.path.join(chats_path, export_item)
+                        try:
+                            backup_conversation(messages, master_plan, console, storage_path=storage_path)
+                        except Exception as e:
+                            print(f"Error: {e}\n")
+                    os.chdir(cwd)
+                elif user_request == ".trim":
                     options = [str(i) for i in range(0, len(messages))]
-                    index_to_edit = await dialogs.getValidOptions(
+                    index_to_trim = await DIALOGS.getValidOptions(
                         default=str(len(messages)-1),
                         options=options,
-                        descriptions=[f"{messages[int(i)]['role']}: {messages[int(i)]['content'][:50]+'...' if len(messages[int(i)]['content'])>50 else messages[int(i)]['content']}" for i in options],
+                        descriptions=[f"{messages[int(i)]['role']}: "+(messages[int(i)]['content'].replace('\n', ' ')[:50]+'...' if len(messages[int(i)]['content'])>50 else messages[int(i)]['content'].replace('\n', ' ')) for i in options],
+                        title="Trim Conversation",
+                        text="Select an entry to be removed:\n(Note: Its paired user/assistant content will also be removed.)"
+                    )
+                    if index_to_trim:
+                        index_to_trim = int(index_to_trim)
+                        trim_role = messages[index_to_trim]["role"]
+                        # make sure the user/assistant is removed in pair; skip system message
+                        if trim_role == "user":
+                            if len(messages) > (index_to_trim + 1) and messages[index_to_trim+1]["role"] == "assistant":
+                                del messages[index_to_trim+1]
+                            del messages[index_to_trim]
+                        elif trim_role == "assistant":
+                            del messages[index_to_trim]
+                            if messages[index_to_trim-1]["role"] == "user":
+                                del messages[index_to_trim-1]
+                elif user_request == ".edit":
+                    options = [str(i) for i in range(0, len(messages))]
+                    index_to_edit = await DIALOGS.getValidOptions(
+                        default=str(len(messages)-1),
+                        options=options,
+                        descriptions=[f"{messages[int(i)]['role']}: "+(messages[int(i)]['content'].replace('\n', ' ')[:50]+'...' if len(messages[int(i)]['content'])>50 else messages[int(i)]['content'].replace('\n', ' ')) for i in options],
                         title="Edit Conversation",
                         text="Select an entry to edit:"
                     )
                     if index_to_edit:
                         index_to_edit = int(index_to_edit)
-                        temp_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp", "edit.md")
-                        writeTextFile(temp_file, messages[index_to_edit]["content"])
-                        edit_file(temp_file)
-                        edited_content = readTextFile(temp_file).strip()
+                        edit_content = messages[index_to_edit]["content"]
+                        if DEFAULT_TEXT_EDITOR == "etextedit":
+                            edited_content = await launch_async(input_text=edit_content, exitWithoutSaving=True, customTitle=f"BibleMate AI")
+                        else:
+                            temp_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp", "edit.md")
+                            writeTextFile(temp_file, edit_content)
+                            edit_file(temp_file)
+                            edited_content = readTextFile(temp_file).strip()
                         if edited_content:
                             messages[index_to_edit]["content"] = edited_content
                             backup_conversation(messages, master_plan) # backup
@@ -605,32 +832,10 @@ Viist https://github.com/eliranwong/biblemate
                     info = f"Lite Context {'Enabled' if config.lite else 'Disabled'}!"
                     display_info(console, info)
                 elif user_request == ".download":
-                    file_ids ={
-                        "bible.db": "1E6pDKfjUMhmMWjjazrg5ZcpH1RBD8qgW",
-                        "collection.db": "1y4txzRzXTBty0aYfFgkWfz5qlHERrA17",
-                        "dictionary.db": "1UxDKGEQa7UEIJ6Ggknx13Yt8XNvo3Ld3",
-                        "encyclopedia.db": "1NLUBepvFd9UDxoGQyQ-IohmySjjeis2-",
-                        "exlb.db": "1Hpo6iLSh5KzgR6IZ-c7KuML--A3nmP1-",
-                    }
-                    file_id = await dialogs.getValidOptions(
-                        options=file_ids.keys(),
-                        title="BibleMate Data Files",
-                        text="Select a file:"
-                    )
-                    if file_id:
-                        output = os.path.join(BIBLEMATEDATA, file_id+".zip")
-                        if os.path.isfile(output):
-                            os.remove(output)
-                        if os.path.isfile(output[:-4]):
-                            os.remove(output[:-4])
-                        gdown.download(id=file_ids[file_id], output=output)
-                        with zipfile.ZipFile(output, 'r') as zip_ref:
-                            zip_ref.extractall(BIBLEMATEDATA)
-                        if os.path.isfile(output):
-                            os.remove(output)
+                    await download_data(console)
                 elif user_request == ".mode":
                     default_ai_mode = "chat" if config.agent_mode is None else "agent" if config.agent_mode else "partner"
-                    ai_mode = await dialogs.getValidOptions(
+                    ai_mode = await DIALOGS.getValidOptions(
                         default=default_ai_mode,
                         options=["agent", "partner", "chat"],
                         descriptions=["AGENT - Fully automated", "PARTNER - Semi-automated, with review and edit prompts", "CHAT - Direct text responses"],
@@ -658,6 +863,30 @@ Viist https://github.com/eliranwong/biblemate
                     config.agent_mode = None
                     write_user_config()
                     display_info(console, f"`Chat` Mode Enabled!")
+                elif user_request == ".defaultbible":
+                    select = await uba_default_bible(options=resource_suggestions_raw["bibleListAbb"], descriptions=resource_suggestions_raw["bibleList"])
+                    if select:
+                        config.default_bible = select
+                        write_user_config()
+                        display_info(console, f"Default bible set to: `{config.default_bible}`")
+                elif user_request == ".defaultcommentary":
+                    select = await uba_default_commentary(options=resource_suggestions_raw["commentaryListAbb"], descriptions=resource_suggestions_raw["commentaryList"])
+                    if select:
+                        config.default_commentary = select
+                        write_user_config()
+                        display_info(console, f"Default commentary set to: `{config.default_commentary}`")
+                elif user_request == ".defaultencyclopedia":
+                    select = await uba_default_encyclopedia(options=resource_suggestions_raw["encyclopediaListAbb"], descriptions=resource_suggestions_raw["encyclopediaList"])
+                    if select:
+                        config.default_encyclopedia = select
+                        write_user_config()
+                        display_info(console, f"Default encyclopedia set to: `{config.default_encyclopedia}`")
+                elif user_request == ".defaultlexicon":
+                    select = await uba_default_lexicon(options=resource_suggestions_raw["lexiconList"])
+                    if select:
+                        config.default_lexicon = select
+                        write_user_config()
+                        display_info(console, f"Default lexicon set to: `{config.default_lexicon}`")
                 elif user_request in (".new", ".exit"):
                     backup_conversation(messages, master_plan, console) # backup
                 # reset
@@ -698,7 +927,7 @@ Viist https://github.com/eliranwong/biblemate
                 if config.agent_mode:
                     this_tool = suggested_tools[0] if suggested_tools else "get_direct_text_response"
                 else: # `partner` mode when config.agent_mode is set to False
-                    this_tool = await dialogs.getValidOptions(options=suggested_tools if suggested_tools else available_tools, title="Suggested Tools", text="Select a tool:")
+                    this_tool = await DIALOGS.getValidOptions(options=suggested_tools if suggested_tools else available_tools, title="Suggested Tools", text="Select a tool:")
                     if not this_tool:
                         this_tool = "get_direct_text_response"
                 # Re-format user request
@@ -831,6 +1060,7 @@ Available tools are: {available_tools}.
                     if not config.agent_mode:
                         info = Markdown("# Review & Confirm\n\nPlease review and confirm the master plan, or make any changes you need.")
                         console.print(info)
+                        print()
                         master_plan_edit = await getTextArea(default_entry=master_plan, title="Review - Master Plan")
                         if not master_plan_edit or master_plan_edit == ".exit":
                             if messages and messages[-1].get("role", "") == "user":
@@ -882,7 +1112,7 @@ Available tools are: {available_tools}.
                 if config.agent_mode:
                     next_tool = suggested_tools[0] if suggested_tools else "get_direct_text_response"
                 else: # `partner` mode when config.agent_mode is set to False
-                    next_tool = await dialogs.getValidOptions(options=suggested_tools if suggested_tools else available_tools, title="Suggested Tools", text="Select a tool:")
+                    next_tool = await DIALOGS.getValidOptions(options=suggested_tools if suggested_tools else available_tools, title="Suggested Tools", text="Select a tool:")
                     if not next_tool:
                         next_tool = "get_direct_text_response"
                 prefix = f"## Next Tool [{step}]\n\n" if DEVELOPER_MODE and not config.hide_tools_order else f"## Tool Selection [{step}]\n\n"
@@ -904,6 +1134,7 @@ Available tools are: {available_tools}.
                 if config.agent_mode == False:
                     info = Markdown("# Review & Confirm\n\nPlease review and confirm the next instruction, or make any changes you need.")
                     console.print(info)
+                    print()
                     next_step_edit = await getTextArea(default_entry=next_step, title="Review - Next Instruction")
                     if not next_step_edit or next_step_edit == ".exit":
                         display_info(console, "I've stopped processing for you.")
@@ -960,9 +1191,6 @@ Please provide me with the final answer to my original request based on the work
             # Backup
             print()
             backup_conversation(messages, master_plan, console)
-
-            if args.exit:
-                break
     
     # back up configurations
     write_user_config(backup=True)
