@@ -1,4 +1,5 @@
-from biblemate import config, BIBLEMATE_VERSION
+from biblemate import config
+from prompt_toolkit.input import create_input
 from prompt_toolkit.layout import Layout, HSplit
 from prompt_toolkit.widgets import Frame, Label
 from prompt_toolkit.styles import Style, merge_styles
@@ -21,6 +22,8 @@ import os
 async def getTextArea(input_suggestions:list=None, default_entry="", title="", multiline:bool=True, completion:Optional[Any]=None, scrollbar:bool=True, read_only:bool=False):
     """Get text area input with a border frame"""
 
+    if config.current_prompt and not default_entry:
+        default_entry = config.current_prompt
     config.current_prompt = "" # reset config.current_prompt
     completer = FuzzyCompleter(WordCompleter(input_suggestions, ignore_case=True)) if input_suggestions else None
     
@@ -53,6 +56,7 @@ async def getTextArea(input_suggestions:list=None, default_entry="", title="", m
         focus_on_click=True,
         wrap_lines=True,
     )
+    text_area.buffer.cursor_position = len(text_area.text)
 
     def unpack_text_chunks(_):
         openai_style = True if config.backend in ("azure", "azure_any", "custom", "deepseek", "github", "github_any", "googleai", "groq", "llamacpp", "mistral", "openai", "xai") else False
@@ -140,7 +144,7 @@ async def getTextArea(input_suggestions:list=None, default_entry="", title="", m
     bindings = KeyBindings()
     config.cursor_position = 0
     
-    if not title: # these shortcuts are irrelevant for review or configuration prompts
+    if not title: # for the main prompt only; these shortcuts are irrelevant for review or configuration prompts
         # help
         @bindings.add("c-y")
         def _(event):
@@ -171,14 +175,15 @@ async def getTextArea(input_suggestions:list=None, default_entry="", title="", m
     @bindings.add("escape", "enter")
     @bindings.add("c-s")
     def _(event):
-        if text_area.text.strip():
-            event.app.exit(result=text_area.text.strip())
+        if not text_area.text.strip():
+            text_area.text = entry = "."
+        event.app.exit(result=text_area.text.strip())
     # submit or new line
     @bindings.add("enter")
     @bindings.add("c-m")
     def _(event):
         entry = text_area.text.strip()
-        if not multiline or (entry.startswith(".") and entry in input_suggestions) or entry.startswith(".open ") or entry.startswith(".load "):
+        if not multiline or (not title and ((entry.strip() == "." or entry.startswith(".") and entry in input_suggestions) or entry.startswith(".open ") or entry.startswith(".import "))):
             event.app.exit(result=text_area.text.strip())
         else:
             text_area.buffer.newline()
@@ -216,6 +221,7 @@ async def getTextArea(input_suggestions:list=None, default_entry="", title="", m
         style=style,
         #clipboard=PyperclipClipboard(), # not useful if mouse_support is not enabled
         #mouse_support=True, # If enabled; content outside the app becomes unscrollable
+        input=create_input(always_prefer_tty=True),
         full_screen=False,
         after_render=unpack_text_chunks if completion is not None else None,
     )
@@ -226,9 +232,10 @@ async def getTextArea(input_suggestions:list=None, default_entry="", title="", m
     # edit in full editor
     while result == ".editprompt":
         if DEFAULT_TEXT_EDITOR == "etextedit":
-            text_area.text = await launch_async(input_text=config.current_prompt, exitWithoutSaving=True, customTitle=f"BibleMate AI [{BIBLEMATE_VERSION}]", startAt=config.cursor_position)
+            text_area.text = await launch_async(input_text=config.current_prompt, exitWithoutSaving=True, customTitle=f"BibleMate AI", startAt=config.cursor_position)
         else:
             text_area.text = edit_temp_file(config.current_prompt)
+        text_area.buffer.cursor_position = len(text_area.text)
         config.current_prompt = ""
         # Run the non-full-screen text area again
         result = await app.run_async()
